#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include "Mesh.h"

#define MESH_XY(mesh,x,y) \
	*(mesh->points + (y * mesh->width) + x)


extern float randomValue();

Mesh *createMesh(int width, int height)
{
  Mesh *newMesh;

  newMesh = (Mesh *)malloc(sizeof(Mesh));

  newMesh->points = (double *)malloc(sizeof(double) * width * height);
  newMesh->width = width;
  newMesh->height = height;

  memset(newMesh->points, 0, sizeof(double) * width * height);


  return newMesh;
}

void freeMesh(Mesh *mesh)
{
	if(!mesh)
		return;
	if(mesh->points)
		free(mesh->points);
	free(mesh);
}

void setMeshColor(Mesh *mesh, double val)
{
	float newColor[4];
	float m;
	int i;
	
	val = fabs(val);

	for(i = 0; i < 4; i++)
	{
		newColor[i] = (((fabs(mesh->highColor[i]) - mesh->lowColor[i]) / mesh->highValue) * fabs(val)) + mesh->lowColor[i];

	}		

	glColor4fv(newColor);
}

void drawMesh(Mesh *mesh)
{
	int x, y;
	double val;
	double delta_x, delta_y;

	delta_x = 1.0 / (float)mesh->width;
	delta_y = 1.0 / (float)mesh->height;

	//A single row is a special case.
	if(mesh->height == 1)
	{
		glBegin(GL_LINE_STRIP);
		for(x = 0; x < mesh->width - 1; x++)
		{
			val = MESH_XY(mesh,x,0);
			//printf("val: %f\n", val);
			setMeshColor(mesh, val);
			glNormal3f(0, 1, 0);
			glVertex3f((float)(x * delta_x),val,(float)(0 * delta_y));
		}
		glEnd();


		return;
	}

	if(mesh->height == 2)
	{
		glBegin(GL_TRIANGLE_STRIP);
		for(x = 0; x < mesh->width - 1; x++)
		{
			val = MESH_XY(mesh,x,0);
			//printf("val: %f\n", val);
			setMeshColor(mesh, val);
			glNormal3f(0, 1, 0);
			glVertex3f((float)(x * delta_x),val,0.01);
			val = 0.0;
			setMeshColor(mesh, val);
			glVertex3f((float)(x * delta_x),val,0.01);
		}
		glEnd();
		return;
	}

	for(y = 0; y < mesh->height - 1; y++)
	{
		glBegin(GL_TRIANGLE_STRIP);
		for(x = 0; x < mesh->width; x++)
		{
			val = MESH_XY(mesh,x,y);
			//printf("val: %f\n", val);
			setMeshColor(mesh, val);
			glNormal3f(0, 1, 0);
			glVertex3f((float)(x * delta_x),val,(float)(y * delta_y));
			val = MESH_XY(mesh,x,(y + 1));
			glNormal3f(0, 1, 0);
			setMeshColor(mesh, val);
			glVertex3f((float)(x * delta_x), val, (float)((y + 1) * delta_y));
		}
		glEnd();
	}

} 

void drawMeshCircle(Mesh *mesh)
{
	int x;
	float val;
	float delta_x, delta_z;
	float theta = 0.0, delta_theta = (float)((float)(2.0 * M_PI) / (float)mesh->width);
	
	//printf("delta_theta: %f\n", delta_theta);

	if(mesh->height == 2)
	{
		glPushMatrix();
		theta = 0;
		glBegin(GL_TRIANGLE_STRIP);
		for(x = 0; x < mesh->width; x++)
		{
			//glRotatef(90, 0, 0, 1);
			//glRotatef(theta, 0, 1, 0); 

			val = MESH_XY(mesh,x,0);
			//printf("val: %f\n", val);
			delta_x = sin(theta);
			delta_z = cos(theta);
			//printf("theta: %f d_x: %f d_z: %f\n", theta, delta_x, delta_z);
			theta += delta_theta;
			setMeshColor(mesh, val);
			glNormal3f(0, 1, 0);
			glVertex3f(delta_x,val,delta_z);
			
			val = 0.0;
			setMeshColor(mesh, val);
			glVertex3f(delta_x, val, delta_z);

		}
		//And connect the ends, just for Jannie. ;)
		theta = 0;
		delta_x = sin(theta);
		delta_z = cos(theta);
		val = MESH_XY(mesh,x,0);
		setMeshColor(mesh, val);
		glNormal3f(0, 1, 0);
		glVertex3f(delta_x,val,delta_z);
		glVertex3f(delta_x, 0, delta_z);

		glEnd();
		glPopMatrix();
		return;
	}

}

float findAverageDerivative(Mesh *mesh, double *lastValues)
{
	int i;
	float prime, accum;	
	static FILE *output = NULL;

	accum = 0;	
	for(i = 0; i < mesh->width; i++)
	{
		prime = (lastValues[i] - mesh->points[i]) / 2.0;	
		accum += prime;
	}


	return fabs(accum);
}

void propagateSine(Mesh *mesh)
{
	int x, y;
	static double time = 0.0;

	for(y = 0; y < mesh->height; y++)
	{ 
		for(x = 0; x < mesh->width; x++)
		{
			MESH_XY(mesh,x,y) = sin((x + time) / (10 * M_PI));
		}
	}

	time += 1.0;
}

void propagateWave(Mesh *mesh)
{
   int x, y;
   for(y = mesh->height - 1; y > 0; y--)
   {
	for(x = 0; x < mesh->width; x++)
	{
		MESH_XY(mesh, x, y) = MESH_XY(mesh, x, y - 1);
	}
   }

}

void drawConnectingMesh(Mesh *one, Mesh *two)
{

	int i;
	float val;
	double delta_x, delta_y;

	delta_x = 1.0 / (float)one->width;
	delta_y = 1.0 / (float)one->height;

	glPushMatrix();
	glBegin(GL_TRIANGLE_STRIP);
	for(i = 0; i < one->width; i++)
	{
		val = MESH_XY(one,i,0) / 5000;
		glVertex3f(((float)(i * delta_x)),val,1.0);
		val = (MESH_XY(two,i,0) / 2500) - 1;
		glVertex3f(((float)(i * delta_x)),val,1.0);
	}
	glEnd();
	glPopMatrix();

}
