/* gluthack.c */
#define UNIX
#ifndef UNIX
#include "winglcrap.h"
#endif

#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <pthread.h>
#ifdef UNIX
#include <sys/times.h>
#else
#include <time.h>
#endif
#include <glib.h>

#include "interface.h"
#include "textures.h"
#include "Mesh.h"
#include "mode.h"
#include "configure.h"
#include <xmms/configfile.h>
#define TIMER 15

float camI, camJ, camK, camTheta;
float camX, camY, camZ;
float camXI, camYJ, camZK;

int drawTimer;
int running = 1;
int currentColorScheme;

char ShowTrails = 0;
char ShowFade = 0;
char glutInitComplete = 0;

float static1;  //Use this to pass data from one state to another.

int winH = 600, winW = 800;

Mesh *leftmesh, *rightmesh, *freqMesh;
Mesh *Meshes[3];
Texture wallTexture;
int glutWindow;
char configFileName[512];

typedef struct point_state {
  int old_x, old_y;
  int new_x, new_y;
  char same_p;
} point_state;


GLuint texture1_ID;
GLuint texture2_ID;
GLuint skyTexture;

char DemoState = 0;	
FILE *configFilePtr;


//Fog Stuff
GLuint filter;						// Which Filter To Use
GLuint fogMode[]= { GL_EXP, GL_EXP2, GL_LINEAR };	// Storage For Three Types Of Fog
GLuint fogfilter= 0;					// Which Fog To Use
GLfloat fogColor[4]= {0.5f, 0.5f, 0.5f, 1.0f};		// Fog Color


static int year = 0;
static int day = 0;


//Thread Stuff
pthread_t displayProc;
pthread_attr_t displayProc_attr;

pthread_mutexattr_t mutexAttr;
pthread_mutex_t pcmDataToDraw, freqDataToDraw;
pthread_mutex_t colorMutex;
pthread_mutex_t doneDrawing;

int drawFlag;
int killFlag;
gint16 pcmData[2][512];
gint16 freqData[2][256];
float highColorArea[3][4];
float lowColorArea[3][4];
int whichWavesForColor;
pthread_t *setupDisplayThread();


//A few declarations
void *DisplayProc(void *arg);

//A little GL Util function
void glRotateDilate(float theta, float r)
{

        float matrix[4][4];
        float a, b;

        a = r * cos(theta);
        b = r * sin(theta);

        matrix[0][0] = a;
        matrix[0][1] = -1 * b;
        matrix[0][2] = 0;
        matrix[0][3] = 0;
        matrix[1][0] = b;
        matrix[1][1] = a;
        matrix[1][2] = 0;
        matrix[1][3] = 0;
        matrix[2][0] = 0;
        matrix[2][1] = 0;
        matrix[2][2] = 1;
        matrix[2][3] = 0;
        matrix[3][0] = 0;
        matrix[3][1] = 0;
        matrix[3][2] = 0;
        matrix[3][3] = 1;

        glMultMatrixf((float *)matrix);
}


pthread_t *setupDisplayThread()
{
   killFlag = 0;
   pthread_mutexattr_init(&mutexAttr);

   if(pthread_mutex_init(&pcmDataToDraw, &mutexAttr) == -1)
   {  
      printf("Error initializing pthread_mutex\n");
   }
   if(pthread_mutex_init(&freqDataToDraw, &mutexAttr) == -1)
   {  
      printf("Error initializing pthread_mutex\n");
   }
   if(pthread_mutex_init(&colorMutex, &mutexAttr) == -1)
   {  
      printf("Error initializing pthread_mutex\n");
   }

   
   memset(highColorArea, 0xff, sizeof(float) * 3 * 4);
   memset(lowColorArea, 0xff, sizeof(float) * 3 * 4);

   pthread_attr_init(&displayProc_attr);
   pthread_create(&displayProc, &displayProc_attr,
                  (void *)&DisplayProc, NULL);

   return &displayProc;
}


void errorCallback(GLenum errorCode)
{
   const GLubyte *estring;

   estring = gluErrorString(errorCode);
   fprintf(stderr, "Quadric Error: %s\n", estring);
}

inline void errorCheck()
{
	GLenum error;

      error = glGetError();
      if(error != GL_NO_ERROR)
	{
		printf("GL ERROR! 0x%x\n", error);
		exit(-1);
	}
      else
	{
		//printf("no gl error\n");
	}
}

void init(void) 
{
  char fname[512];
   GLfloat LightAmbient[]  = { 1.0f, 1.0, 1.0, 1.0};
  GLfloat LightDiffuse[]  = { 1.0, 1.0, 1.0, 1.0};
  GLfloat LightSpecular[] = { 1.0, 1.0, 1.0, 1.0};
  GLfloat LightPosition[] = { 5.0, 5.0, -5.0, 1.0};
  GLfloat MatSpecular[] = {1.0, 1.0, 1.0, 1.0 };
  GLfloat MatShininess[] = { 50.0 };
  GLenum error;

  Texture texStruct;
  //glViewport(0,0, widget->allocation.width, widget->allocation.height);

  glClearColor(0.0, 0.0, 0.0, 1.0);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_COLOR_MATERIAL);
  glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);      

 glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, MatSpecular); 
 //glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, MatShininess);

      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      //glOrtho(-50.0, 50.0, -50.0, 50.0, -50.0, 50.0);
      //glTranslatef(0.0, 0.0, 25.0);
      //glOrtho(-1.0, 1.0, -1.0, 1.0, -10.0, 10.0);
      glMatrixMode(GL_MODELVIEW);
//      glEnable(GL_LIGHTING);  //turn on lights in general.

	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);
	glLightfv(GL_LIGHT0, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, LightSpecular);
      	glEnable(GL_LIGHT0);

      glEnable(GL_DEPTH_TEST);
errorCheck();
      glEnable(GL_BLEND);
errorCheck();
      //glEnable(GL_ALPHA);
errorCheck();
      glEnable(GL_NORMALIZE);
errorCheck();
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
errorCheck();
      glAlphaFunc(GL_GEQUAL, 0.05);
errorCheck();
      //glPointSize(4);
      glLineWidth(1);

     errorCheck();
      
      camI = camJ = camK = camTheta = 30;      
      camX = 0;camY = 0; camZ = 150;

      drawFlag = 0;
}

void update(int fnord)
{

  glutPostRedisplay ();
  glutTimerFunc (TIMER, update, 0);

}

void drawStatusDisplay(float fps, float delta_t, float camI, float camJ)
{
/*
  int size;
  float pink[] = {1, 0, 0.5};
  char str[1024];


  fontSize (14);
  fontShadow ();
  fontShadowColor (0.0,0.0,0.0);
  sprintf(str, "\\a(255 0 0 255)%f \\a(0 175 0 255)fps", fps);
  fontDrawString (0,winH - 20, str);
*/	
}

float randomValue()
{
	float ret;
	int i = random();
	
	ret = (float)((float)(i % 100)) / 100.0;
	ret -= 0.5;
	//printf("%f\n", ret);

	return 0;

}

void displayKalied(void)
{
  static GLUquadricObj *quad = NULL;
  int i; 
  int x, y;
  static int frames = 0;
  static float rotSum = 0.0;
  static float fps;
  float average;
  static char reps = 0;
  float delta_t = 0.0;
  static float timer;
  static float seconds;
  static float leftRot = 0.0, rightRot = 0.0;
  static double *rightHistory = NULL, *leftHistory = NULL;
  GLuint freqList;

  frames++;

  if(killFlag)
  {
	freeMesh(freqMesh);
	freeMesh(leftmesh);
	freeMesh(rightmesh);
	/*
	if(!pthread_mutex_destroy(&pcmDataToDraw))
		printf("Error releasing mutex\n");
	if(!pthread_mutex_destroy(&freqDataToDraw))
		printf("Error releasing mutex\n");
	if(!pthread_mutex_destroy(&colorMutex))
		printf("Error releasing mutex\n");
	if(!pthread_mutex_destroy(&doneDrawing))
		printf("Error releasing mutex\n");
	*/
        glutDestroyWindow(glutWindow);
	pthread_exit(NULL);
  }

  if(modeSwitched)
  {
	modeSwitched = 0;
	glutDisplayFunc(displayFuncs[modeFlag]);
  }

  if(!rightHistory)
  {
	rightHistory = (double *)malloc(sizeof(double) * rightmesh->width);
	memset(rightHistory, 0, sizeof(double) * rightmesh->width);
  }
  if(!leftHistory)
  {
	leftHistory = (double *)malloc(sizeof(double) * leftmesh->width);
	memset(leftHistory, 0, sizeof(double) * leftmesh->width);
  }

  
  delta_t = times(NULL) - timer;
  delta_t /= 50;
  timer = times(NULL);
  if (((timer / 100) - seconds) > 1)
    {
      seconds = (timer / 100);
      fps = (float)frames;
      frames = 0;
    }
  //printf("delta_t: %f %f %f %f\r", 
  //delta_t, timer, seconds, fps); fflush(stdout);


glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  gluLookAt(0, 15, 0,
	    0, 0, 0,
	    0, 0, 1);


  if (quad == NULL)
    {
      quad = gluNewQuadric();
      gluQuadricTexture(quad, GL_TRUE);
     }

  if((drawFlag) && (running) && (frames != 0))   //There's new data to draw.
  {
     pthread_mutex_lock(&pcmDataToDraw);
     drawFlag = 0;

     average = 0;
     memset(rightmesh->points, 0, sizeof(double) * rightmesh->width * rightmesh->height);
     for(x = 0; x < 128; x++ )
     {
	for(y = 0; y < rightmesh->height; y++)
	{
		*(rightmesh->points + (y * rightmesh->width) + x) += pcmData[0][x * 4];
		*(leftmesh->points + (y * leftmesh->width) + x) = pcmData[1][x * 4];
		//printf("%d\n", pcmData[0][x]);
		//fflush(stdout);
	}
     }
     pthread_mutex_unlock(&pcmDataToDraw);

     pthread_mutex_lock(&freqDataToDraw);
     for(x = 0; x < 32; x++)
     {
        for(y = 0; y < freqMesh->height; y++)
	{
		*(freqMesh->points + (y * freqMesh->width) + x) = freqData[1][x] * 5;
	}
     }
     pthread_mutex_unlock(&freqDataToDraw);

     //propagateWave(leftmesh);
     //propagateWave(rightmesh);

	//printf("glut locking\n"); fflush(stdout);
	//if(pthread_mutex_trylock(&colorMutex))
	{
		//printf("glut locked\n"); fflush(stdout);
		for(i = 0; i < 3; i++)
		{
			//if(highColorArea[i][0] >= 0)
			{
				switch (i)
				{
				  case 0:
					leftmesh->highColor[0] = highColorArea[i][0] = randomValue();
					leftmesh->highColor[1] = highColorArea[i][1] += randomValue();
					leftmesh->highColor[2] = highColorArea[i][2] += randomValue();
					leftmesh->highColor[3] = 1;
					leftmesh->lowColor[0] = lowColorArea[i][0] += randomValue();
					leftmesh->lowColor[1] = lowColorArea[i][1] += randomValue();
					leftmesh->lowColor[2] = lowColorArea[i][2] += randomValue();
					leftmesh->lowColor[3] = 1;
					break;
				  case 1:
					freqMesh->highColor[0] = highColorArea[i][0] += randomValue();
					freqMesh->highColor[1] = highColorArea[i][1] += randomValue();
					freqMesh->highColor[2] = highColorArea[i][2] += randomValue();
					freqMesh->highColor[3] = 1;
					freqMesh->lowColor[0] = lowColorArea[i][0] += randomValue();
					freqMesh->lowColor[1] = lowColorArea[i][1] += randomValue();
					freqMesh->lowColor[2] = lowColorArea[i][2] += randomValue();
					freqMesh->lowColor[3] = 1;
					break;
		 		  case 2:
					rightmesh->highColor[0] = highColorArea[i][0] += randomValue();
					rightmesh->highColor[1] = highColorArea[i][1] += randomValue();
					rightmesh->highColor[2] = highColorArea[i][2] += randomValue();
					rightmesh->highColor[3] = 1;
					rightmesh->lowColor[0] = lowColorArea[i][0] += randomValue();
					rightmesh->lowColor[1] = lowColorArea[i][1] += randomValue();
					rightmesh->lowColor[2] = lowColorArea[i][2] += randomValue();
					rightmesh->lowColor[3] = 1;
					//memcpy(rightmesh->highColor, highColorArea[i], sizeof(float) * 4);
					//memcpy(rightmesh->lowColor, lowColorArea[i], sizeof(float) * 4);
					break;
				
				}		
			}
		}	
		
		//pthread_mutex_unlock(&colorMutex);
		//printf("glut unlocked\n"); fflush(stdout);

	}
     //printf("%d\n", rotSum); fflush(stdout);
  }  

  //glRotatef(90, 1, 0, 0);
  //glRotatef(camI, 1, 0, 0);
  //glRotatef(camJ, 0, 0, 1);

  glColor3f(1, 1, 1);

  rotSum += 0.25;

/*
  //Draw the hoops.
  //rightRot += ((rand() % 100) / 200) - 0.5;
  //leftRot += ((rand() % 100) / 200) - 0.5;
  rightRot = (findAverageDerivative(rightmesh, rightHistory) / 12000.0);
  leftRot = (findAverageDerivative(leftmesh, leftHistory) / 12000.0);
  glPushMatrix();
  glRotatef(-1 * rotSum, 0, 1, 0);
  glRotatef(rightRot, 0, 0, 1);
  //glScalef(rightRot, 0.00007, rightRot);
  //printf("%f\n", rightRot);
  glScalef(4.2, 0.00007, 4.2); //0.5);
  drawMeshCircle(rightmesh);
  glPopMatrix();
 
  glPushMatrix();
  //glRotatef(rotSum, 0, 1, 0);
  glRotatef(leftRot, 1, 0, 0);
  glScalef(2.1, 0.00007, 2.1); //0.5);
  drawMeshCircle(leftmesh);
  glPopMatrix();

  memcpy(rightHistory, rightmesh->points, sizeof(double) * rightmesh->width);
  memcpy(leftHistory, leftmesh->points, sizeof(double) * leftmesh->width);
*/

  glPushMatrix();
  glRotatef(rotSum, 0, 1, 0);
  //Draw the central flames
  for(i = 0; i < 4; i++)
  {
	glRotatef(i * (360 / 8), 0, 1, 0);
	if(i == 0)
        {
	freqList = glGenLists(1);
 	glNewList(freqList, GL_COMPILE);
	glPushMatrix();	
  	for(x = 0; x < 2; x++)
  	{
	  glPushMatrix();
	  //glTranslatef(0, 0, 1);
	  glScalef(4.5, 0.00004, 0.5);

  	drawMesh(freqMesh);
  	glRotatef(180, 1, 0, 0);

  	drawMesh(freqMesh);
	  glRotatef(180, 0, 0, 1);

	  drawMesh(freqMesh);
	  glRotatef(180, 1, 0, 0);

	  drawMesh(freqMesh);
	  glRotatef(90, 0, 0, 1);

	  glPopMatrix();
	}	
	glPopMatrix();
	glEndList();
	}
	//glRotateDilate(45, 2);
	//glCallList(freqList);
	glPushMatrix();
	glRotatef(90, 1, 0, 0);

	//glRotateDilate(45, 2);
	glCallList(freqList);
	glPopMatrix();
  }
  glPopMatrix();

   glDeleteLists(freqList, 1);


//drawStatusDisplay(fps, delta_t, day, year);
  //printf("fps: %f\r", fps);
//  errorCheck();
  glutSwapBuffers();
  
}


void display_003(void)
{
  static GLUquadricObj *quad = NULL;
  int i; 
  int x, y;
  static int frames = 0;
  static float rotSum = 0.0;
  static float fps;
  float average;
  static char reps = 0;
  float delta_t = 0.0;
  static float timer;
  static float seconds;
  static float leftRot = 0.0, rightRot = 0.0;
  static double *rightHistory = NULL, *leftHistory = NULL;
  GLuint freqList;

  frames++;

  if(killFlag)
  {
	freeMesh(freqMesh);
	freeMesh(leftmesh);
	freeMesh(rightmesh);
	/*
	if(!pthread_mutex_destroy(&pcmDataToDraw))
		printf("Error releasing mutex\n");
	if(!pthread_mutex_destroy(&freqDataToDraw))
		printf("Error releasing mutex\n");
	if(!pthread_mutex_destroy(&colorMutex))
		printf("Error releasing mutex\n");
	if(!pthread_mutex_destroy(&doneDrawing))
		printf("Error releasing mutex\n");
	*/
        glutDestroyWindow(glutWindow);
	pthread_exit(NULL);
  }

  if(modeSwitched)
  {
	modeSwitched = 0;
	glutDisplayFunc(displayFuncs[modeFlag]);
  }

  if(!rightHistory)
  {
	rightHistory = (double *)malloc(sizeof(double) * rightmesh->width);
	memset(rightHistory, 0, sizeof(double) * rightmesh->width);
  }
  if(!leftHistory)
  {
	leftHistory = (double *)malloc(sizeof(double) * leftmesh->width);
	memset(leftHistory, 0, sizeof(double) * leftmesh->width);
  }

  
  delta_t = times(NULL) - timer;
  delta_t /= 50;
  timer = times(NULL);
  if (((timer / 100) - seconds) > 1)
    {
      seconds = (timer / 100);
      fps = (float)frames;
      frames = 0;
    }
  //printf("delta_t: %f %f %f %f\r", 
  //delta_t, timer, seconds, fps); fflush(stdout);


glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  gluLookAt(0, 0, 14,
	    0, 0, 0,
	    0, 1, 0);


  if (quad == NULL)
    {
      quad = gluNewQuadric();
      gluQuadricTexture(quad, GL_TRUE);
     }

  if((drawFlag) && (running) && (frames != 0))   //There's new data to draw.
  {
     pthread_mutex_lock(&pcmDataToDraw);
     drawFlag = 0;

     average = 0;
     memset(rightmesh->points, 0, sizeof(double) * rightmesh->width * rightmesh->height);
     for(x = 0; x < 128; x++ )
     {
	for(y = 0; y < rightmesh->height; y++)
	{
		*(rightmesh->points + (y * rightmesh->width) + x) += pcmData[0][x * 4];
		*(leftmesh->points + (y * leftmesh->width) + x) = pcmData[1][x * 4];
		//printf("%d\n", pcmData[0][x]);
		//fflush(stdout);
	}
     }
     pthread_mutex_unlock(&pcmDataToDraw);

     pthread_mutex_lock(&freqDataToDraw);
     for(x = 0; x < 32; x++)
     {
        for(y = 0; y < freqMesh->height; y++)
	{
		*(freqMesh->points + (y * freqMesh->width) + x) = freqData[1][x] * 5;
	}
     }
     pthread_mutex_unlock(&freqDataToDraw);

     //propagateWave(leftmesh);
     //propagateWave(rightmesh);

	//printf("glut locking\n"); fflush(stdout);
	//if(pthread_mutex_trylock(&colorMutex))
	{
		//printf("glut locked\n"); fflush(stdout);
		for(i = 0; i < 3; i++)
		{
			//if(highColorArea[i][0] >= 0)
			{
				switch (i)
				{
				  case 0:
					leftmesh->highColor[0] = highColorArea[i][0] = randomValue();
					leftmesh->highColor[1] = highColorArea[i][1] += randomValue();
					leftmesh->highColor[2] = highColorArea[i][2] += randomValue();
					leftmesh->highColor[3] = 1;
					leftmesh->lowColor[0] = lowColorArea[i][0] += randomValue();
					leftmesh->lowColor[1] = lowColorArea[i][1] += randomValue();
					leftmesh->lowColor[2] = lowColorArea[i][2] += randomValue();
					leftmesh->lowColor[3] = 1;
					break;
				  case 1:
					freqMesh->highColor[0] = highColorArea[i][0] += randomValue();
					freqMesh->highColor[1] = highColorArea[i][1] += randomValue();
					freqMesh->highColor[2] = highColorArea[i][2] += randomValue();
					freqMesh->highColor[3] = 1;
					freqMesh->lowColor[0] = lowColorArea[i][0] += randomValue();
					freqMesh->lowColor[1] = lowColorArea[i][1] += randomValue();
					freqMesh->lowColor[2] = lowColorArea[i][2] += randomValue();
					freqMesh->lowColor[3] = 1;
					break;
		 		  case 2:
					rightmesh->highColor[0] = highColorArea[i][0] += randomValue();
					rightmesh->highColor[1] = highColorArea[i][1] += randomValue();
					rightmesh->highColor[2] = highColorArea[i][2] += randomValue();
					rightmesh->highColor[3] = 1;
					rightmesh->lowColor[0] = lowColorArea[i][0] += randomValue();
					rightmesh->lowColor[1] = lowColorArea[i][1] += randomValue();
					rightmesh->lowColor[2] = lowColorArea[i][2] += randomValue();
					rightmesh->lowColor[3] = 1;
					//memcpy(rightmesh->highColor, highColorArea[i], sizeof(float) * 4);
					//memcpy(rightmesh->lowColor, lowColorArea[i], sizeof(float) * 4);
					break;
				
				}		
			}
		}	
		
		//pthread_mutex_unlock(&colorMutex);
		//printf("glut unlocked\n"); fflush(stdout);

	}
     //printf("%d\n", rotSum); fflush(stdout);
  }  

  glRotatef(camI, 1, 0, 0);
  glRotatef(camJ, 0, 1, 0);

  glColor3f(1, 1, 1);

  rotSum += 0.25;

  //Draw the hoops.
  //rightRot += ((rand() % 100) / 200) - 0.5;
  //leftRot += ((rand() % 100) / 200) - 0.5;
  rightRot = (findAverageDerivative(rightmesh, rightHistory) / 12000.0);
  leftRot = (findAverageDerivative(leftmesh, leftHistory) / 12000.0);
  glPushMatrix();
  glRotatef(-1 * rotSum, 0, 1, 0);
  glRotatef(rightRot, 0, 0, 1);
  //glScalef(rightRot, 0.00007, rightRot);
  //printf("%f\n", rightRot);
  glScalef(4.2, 0.00007, 4.2); //0.5);
  drawMeshCircle(rightmesh);
  glPopMatrix();
 
  glPushMatrix();
  //glRotatef(rotSum, 0, 1, 0);
  glRotatef(leftRot, 1, 0, 0);
  glScalef(2.1, 0.00007, 2.1); //0.5);
  drawMeshCircle(leftmesh);
  glPopMatrix();

  memcpy(rightHistory, rightmesh->points, sizeof(double) * rightmesh->width);
  memcpy(leftHistory, leftmesh->points, sizeof(double) * leftmesh->width);

  glPushMatrix();
  glRotatef(rotSum, 0, 1, 0);
  //Draw the central flames
  for(i = 0; i < 4; i++)
  {
	glRotatef(i * (360 / 8), 0, 1, 0);
	if(i == 0)
        {
	freqList = glGenLists(1);
 	glNewList(freqList, GL_COMPILE);
	glPushMatrix();	
  	for(x = 0; x < 2; x++)
  	{
	  glPushMatrix();
	  //glTranslatef(0, 0, 1);
	  glScalef(4.5, 0.00004, 0.5);

  	drawMesh(freqMesh);
  	glRotatef(180, 1, 0, 0);

  	drawMesh(freqMesh);
	  glRotatef(180, 0, 0, 1);

	  drawMesh(freqMesh);
	  glRotatef(180, 1, 0, 0);

	  drawMesh(freqMesh);
	  glRotatef(90, 0, 0, 1);

	  glPopMatrix();
	}	
	glPopMatrix();
	glEndList();
	}
	//glRotateDilate(45, 2);
	glCallList(freqList);
	//glPushMatrix();
	//glRotatef(90, 1, 0, 0);

	//glRotateDilate(45, 2);
	//glCallList(freqList);
	//glPopMatrix();
  }
  glPopMatrix();

   glDeleteLists(freqList, 1);


//drawStatusDisplay(fps, delta_t, day, year);
  //printf("fps: %f\r", fps);
//  errorCheck();
  glutSwapBuffers();
}
  

void reshape (int w, int h)
{
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (w <= h)
     gluPerspective(30.0f, (float)((float)w / (float)h), 1.0, 4250.0);
   else
    gluPerspective(30.0f, (float)((float)w / (float)h), 1.0, 1000.0);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();


}

void loadColorFromFile(int schemeNum)
{
	FILE *data;
	int i, j;
	char fname[512];
	char buf[2048];
	char *bufPtr;
		
   	sprintf(fname, "%s/ZoN.rc", TEXTURE_DIR);
	data = fopen(fname, "r");
	for(i = 0; i != schemeNum;)
	{
	  if(feof(data))
		{
			return;
		}
	
	   memset(buf, 0, 2048);
	   fgets(buf, 2048, data);
	  if(strncmp("##", buf, 2) == 0)
	  {
		i++;
	  }
	}

	for(i = 0; i < 3; i++)
	{
		memset(buf, 0, 2048);
		fgets(buf, 2048, data);
		bufPtr = buf;
		for(j = 0; j < 4; j++)
		{
			highColorArea[i][j] = (float)atof(bufPtr);	
			while(*bufPtr != ' ')
				bufPtr++;
			while(*bufPtr == ' ')
				bufPtr++;
		}
		for(j = 0; j < 4; j++)
		{
			lowColorArea[i][j] = (float)atof(bufPtr);	
			while(*bufPtr != ' ')
				bufPtr++;
			while(*bufPtr == ' ')
				bufPtr++;
		}
	}
	
	if(data)
		fclose(data);	

	for(i = 0; i < 3; i++)
	{
		for(j = 0; j < 4; j++)
		{
			printf("%f  ", highColorArea[i][j]);
		}
		for(j = 0; j < 4; j++)
		{
			printf("%f  ", lowColorArea[i][j]);
		}
		printf("\n");
	}

	currentColorScheme = schemeNum;
}

void keyboardFunc(unsigned char key, int x, int y)
{
	int i, j;

   if(((char)key - 48 > 0) && ((char)key - 48 < 10))
	{
		glLineWidth((int)(key - 48));
		return;
	}

   switch (key) {
   case 27:
   case 'q':
	killFlag = 1;
     break;
   case 'P':
     running = (running + 1) % 2;
     printf("%d\n", running);
     fflush(stdout);
     break;
   case 'f':
	glutFullScreen();
	   break;
   case 'g':
	//glutGameModeString("640x480:16@60");
	glutGameModeString("800x600");
	//glutEnterGameMode();
	break;
   case ' ':
/*	
	for(i = 0; i < 3; i++)
	{
		for(j = 0; j < 4; j++)
		{
			fprintf(configFilePtr, "%f  ", highColorArea[i][j]);
		}
		for(j = 0; j < 4; j++)
		{
			fprintf(configFilePtr, "%f  ", lowColorArea[i][j]);
		}
		fprintf(configFilePtr, "\n");
	}
	fprintf(configFilePtr, "##\n");
	fflush(configFilePtr);
*/
     break;
   case 'R':
		camI = camJ = camK = 0.0;
		camX = 0;camY = -25; camZ = -150;
		break;
   case 'w':
	   camZ += 3;
	   break;
   case 's':
	   camZ -= 3;
	   break;
   case 'a':
	   camX += 3;
	   break;
   case 'd':
	   loadColorFromFile(0);
	   break;
   case 'n':
	   loadColorFromFile(currentColorScheme + 1);
	   break;
   case 'p':
	   loadColorFromFile(currentColorScheme - 1);
	   break;
   case 0x08:
	   camY -= 3;
	   break;
   default:
     printf("0x%.02x\n", key);
     fflush(stdout);
     break;
   }
}

void mouseMove(int x, int y)
{
	static int oldX = -1, oldY = -1;

	if ( (abs(x - oldX) < 50) && (abs(y - oldY) < 50) )
	  {

	    if(oldX >= 0) 
	      {
		//Rotate about the Y axis.
		camJ -= 0.5 * (oldX - x);
	      }
	    if (oldY >= 0)
	      {
		//rotate about the X axis.
		camI -= 0.5 * (oldY - y);
	      }
	    
	    camTheta = sqrt((oldX - x) * (oldX - x) + (oldY - y) * (oldY - y));
	  }

	

	oldX = x;
	oldY = y;
}


//;HERB
void menu(int value)
{

  if (value == -1)
	return keyboardFunc('P', 0, 0);

  printf("menu: entry = %d\n", value); fflush(stdout);

  switchMode(value);

}

void menu2(int value)
{
	char *string;

	string = (char *)value;
}

void menu3(int value)
{

}

void changeMeshColor(int meshNum, float highColor[4], float lowColor[4])
{
   memcpy(Meshes[meshNum]->highColor, highColor, sizeof(float) * 4);
   memcpy(Meshes[meshNum]->lowColor, lowColor, sizeof(float) * 4);
}

void *DisplayProc(void *arg)
{
  int argc = 1;
  char *argv[] = {"-=< ZoN >=-"};
  int win, mainmenu, submenu, submenu2;	
  gchar* conffile;
  ConfigFile *cfg;
  int i;
  char lowname[10];
  char highname[10]; 
  if (!glutInitComplete)
	{ 
	  glutInit(&argc, argv);
	  glutInitComplete = 1;
	}

  	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  	glutInitWindowSize(winW, winH); 
	glutInitWindowPosition(0, 0);
	glutWindow = win = glutCreateWindow(argv[0]);
	//glutGameModeString("800x600");
	//glutEnterGameMode();

  leftmesh = createMesh(128, 2);
  leftmesh->highValue = 1000;
  leftmesh->highColor[0] = 1.0;
  leftmesh->highColor[1] = 1.0;
  leftmesh->highColor[2] = 1.0;
  leftmesh->highColor[3] = 0.2;
  leftmesh->lowColor[0] = 0.0;
  leftmesh->lowColor[1] = 0.0;
  leftmesh->lowColor[2] = 1.0;
  leftmesh->lowColor[3] = 1.0;

  rightmesh = createMesh(128, 2);
  rightmesh->highValue = 1000;
  rightmesh->highColor[0] = 1.0;
  rightmesh->highColor[1] = 1.0;
  rightmesh->highColor[2] = 1.0;
  rightmesh->highColor[3] = 0.2;
  rightmesh->lowColor[0] = 0.0;
  rightmesh->lowColor[1] = 0.0;
  rightmesh->lowColor[2] = 1.0;
  rightmesh->lowColor[3] = 1.0;

  freqMesh = createMesh(32, 2);
  freqMesh->highValue = 100;
  freqMesh->highColor[0] = 0.0;
  freqMesh->highColor[1] = 0.7;
  freqMesh->highColor[2] = 0.3;
  freqMesh->highColor[3] = 1.0;
  freqMesh->lowColor[0] = 0.0;
  freqMesh->lowColor[1] = 0.0;
  freqMesh->lowColor[2] = 1.0;
  freqMesh->lowColor[3] = 1.0;

  Meshes[0] = leftmesh;
  Meshes[1] = freqMesh;
  Meshes[2] = rightmesh;
  
  init();
	
   //;HERB
   registerDisplayFunc(displayKalied);
   registerDisplayFunc(display_003);
   switchMode(desiredMode);
   modeSwitched = 0;

  /*
  mainmenu = glutCreateMenu(menu);
  glutAddMenuEntry("Mode 0", 0);
  glutAddMenuEntry("Mode 1", 1);
  glutAttachMenu(GLUT_RIGHT_BUTTON);
  */

  //;HERB
  glutDisplayFunc(displayKalied); 
  glutTimerFunc (TIMER, update, 0); 
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboardFunc);
  glutMotionFunc(mouseMove);

  loadColorFromFile(0);  
  conffile = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
  cfg = xmms_cfg_open_file(conffile);
  if (!cfg)
	  cfg = xmms_cfg_new();
  
  if (!xmms_cfg_read_int(cfg,"ZoN","Mode", &desiredMode))
	switchMode(0);
  else
	  switchMode(desiredMode);

  for(i = 0; i < 4; i++)
  {
	   sprintf(lowname,"low%d",i);
	   sprintf(highname,"high%d",i);
	   xmms_cfg_read_float(cfg,"ZoN",lowname,&(lowColorArea[1][i]));
	   xmms_cfg_read_float(cfg,"ZoN",highname,&(highColorArea[1][i]));
  }
  
  xmms_cfg_free(cfg);
  g_free(conffile); 


  srandom(time(NULL));

  glutMainLoop();
  return 0;
}
